#include <cstddef>
#include <cstdint>
#include <map>
#include <print>
#include <tuple>
#include <vector>

#include <pw/core/image.hpp>

namespace pw {

struct image_layout {

    enum pixel_layout { RGB8, RGBA8, LUM8, DEPTH };

    static constexpr auto bits_per_channel(pixel_layout layout) {
        constexpr static auto lut =
            std::array{std::make_tuple(RGB8, sizeof(std::uint8_t) << 3),
                       std::make_tuple(RGBA8, sizeof(std::uint8_t) << 3),
                       std::make_tuple(LUM8, sizeof(std::uint8_t) << 3),
                       std::make_tuple(DEPTH, sizeof(float) << 3)};

        const auto iter = std::find_if(
            std::begin(lut), std::end(lut),
            [&layout](const auto& v) { return std::get<0>(v) == layout; });

        return iter != std::end(lut) ? std::get<1>(*iter) : 0u;
    }
};

} // namespace pw

auto main() -> int {

    auto val = pw::image_layout::bits_per_channel(
        pw::image_layout::pixel_layout::RGB8);

    std::print("RGB8 = {}bits per channel\n", val);
    std::print("DEPTH = {}bits per channel\n",
               pw::image_layout::bits_per_channel(
                   pw::image_layout::pixel_layout::DEPTH));
    std::print("LUM8 = {}bits per channel\n",
               pw::image_layout::bits_per_channel(
                   pw::image_layout::pixel_layout::LUM8));

    // pw::image
}
